@@ -17,16 +17,59 @@
  * --------------------------------------------------------------------------
  */
 
-#include <atomic>
+#include <glibmm.h>
 #include "gx_common.h"
 
+/****************************************************************
+ ** "atomic" value access
+ */
+
+inline void atomic_set(volatile int32_t* p, int32_t v)
+{
+  g_atomic_int_set(p, v);
+}
+
+inline int32_t atomic_get(volatile int32_t& p)
+{
+  return g_atomic_int_get(&p);
+}
+
+inline bool atomic_compare_and_exchange(volatile int32_t *p, int32_t oldv, int32_t newv)
+{
+  return g_atomic_int_compare_and_exchange(p, oldv, newv);
+}
+
+template <class T>
+inline void atomic_set(T **p, T *v)
+{
+  g_atomic_pointer_set(p, v);
+}
+
+template <class T>
+inline void atomic_set_0(T **p)
+{
+  g_atomic_pointer_set(p, 0);
+}
+
+template <class T>
+inline T *atomic_get(T*& p)
+{
+  return static_cast<T*>(g_atomic_pointer_get(&p));
+}
+
+template <class T>
+inline bool atomic_compare_and_exchange(T **p, T *oldv, T *newv)
+{
+  return g_atomic_pointer_compare_and_exchange(reinterpret_cast<void* volatile*>(p), oldv, newv);
+}
+
+
 #include "gxamp.h"
 #include "gx_resampler.h"
 #include "gx_convolver.h"
 #include "gx_pluginlv2.h"   // define struct PluginLV2
 #include "impulse_former.h"
 #include "ampulse_former.h"
-#include "gx_bypass.cc"     // bypass ramping
 
 #ifndef __SSE__
 #include "noiser.cc"
@@ -58,7 +101,6 @@ declare(gxamp15);
 declare(gxamp16);
 declare(gxamp17);
 declare(gxamp18);
-declare(gxnoamp);
 
 static plug amp_model [] = {
     gxamp::plugin, //0
@@ -83,8 +125,6 @@ static plug amp_model [] = {
     gxamp8::plugin, //15
     gxamp16::plugin, //16
     gxamp6::plugin, //17
-
-    gxnoamp::plugin, //18
 };
 
 static const size_t AMP_COUNT = sizeof(amp_model) / sizeof(amp_model[0]);
@@ -154,9 +194,6 @@ static const size_t TS_COUNT = sizeof(tonestack_model) / sizeof(tonestack_model[
 class GxPluginMono
 {
 private:
-  GxBypass                     bp;
-  float*                       bypass;
-  DenormalProtection           MXCSR;
   // internal stuff
   float*                       output;
   float*                       input;
@@ -192,27 +229,28 @@ private:
   float                        *alevel;
   float                        alevel_;
   float                        pre;
+  float                        val;
   bool                         doit;
   float*                       schedule_ok;
   float                        schedule_ok_;
-  std::atomic<bool>            _execute;
+  volatile int32_t             schedule_wait;
 
   inline bool cab_changed() 
-    {return std::abs(cab - clevel_ ) > 0.1;}
+    {return abs(cab - (clevel_ + c_model_)) > 0.1;}
   inline bool buffsize_changed() 
-    {return bufsize != cur_bufsize;}
-  inline bool IsPowerOfTwo(uint32_t x)
-    {return (x >= 64) && ((x & (x - 1)) == 0);}
+    {return abs(bufsize - cur_bufsize) != 0;}
   inline void update_cab() 
-    {cab = clevel_ ; c_old_model_ = c_model_;}
+    {cab = (clevel_ + c_model_); c_old_model_ = c_model_;}
   inline bool change_cab() 
-    {return std::abs(c_old_model_ - c_model_) > 0.1;}
+    {return abs(c_old_model_ - c_model_) > 0.1;}
   inline bool pre_changed() 
-    {return std::abs(pre - alevel_) > 0.1;}
+    {return abs(pre - alevel_) > 0.1;}
   inline void update_pre() 
     {pre = (alevel_);}
   inline bool val_changed() 
-    {return  std::abs(alevel_ - (*alevel)) > 0.1 || abs(clevel_ - (*clevel)) > 0.1 || std::abs(c_model_ - (*c_model)) > 0.1;}
+    {return abs(val - ((*alevel) + (*clevel) + (*c_model))) > 0.1;}
+  inline void update_val() 
+    {val = (alevel_) + (clevel_) + (c_model_);}
 
   // LV2 stuff
   LV2_URID_Map*                map;
@@ -255,12 +293,8 @@ public:
   ~GxPluginMono();
 };
 
-
   // constructor
 GxPluginMono::GxPluginMono() :
-  bp(),
-  bypass(0),
-  MXCSR(),
   output(NULL),
   input(NULL),
   s_rate(0),
@@ -284,10 +318,11 @@ GxPluginMono::GxPluginMono() :
   alevel(NULL),
   alevel_(0),
   pre(0),
+  val(0),
   schedule_ok(NULL),
   schedule_ok_(0)
 {
-
+  atomic_set(&schedule_wait,0);
 };
   // destructor
 GxPluginMono::~GxPluginMono()
@@ -302,8 +337,8 @@ GxPluginMono::~GxPluginMono()
 
 void GxPluginMono::do_work_mono()
 {
-  if (buffsize_changed() && IsPowerOfTwo(cur_bufsize)) 
-  {
+  if (buffsize_changed()) 
+   {
      printf("buffersize changed to %u\n",cur_bufsize);
      if (cabconv.is_runnable())
         {
@@ -316,32 +351,28 @@ void GxPluginMono::do_work_mono()
           ampconv.stop_process();
         }
      bufsize = cur_bufsize;
+
 	 cabconv.cleanup();
      CabDesc& cab = *getCabEntry(static_cast<uint32_t>(c_model_)).data;
      cabconv.cab_count = cab.ir_count;
      cabconv.cab_sr = cab.ir_sr;
+     cabconv.cab_data = cab.ir_data;
      cabconv.set_samplerate(s_rate);
      cabconv.set_buffersize(bufsize);
-     float cab_irdata_c[cabconv.cab_count];
-     float adjust_1x8 = 1;
-     if ( c_model_ == 17.0) adjust_1x8 = 0.5;
-     impf.compute(cabconv.cab_count, cabconv.cab_data, cab_irdata_c, (clevel_ * adjust_1x8) );
-     cabconv.cab_data_new = cab_irdata_c;
-     cabconv.configure(cabconv.cab_count, cabconv.cab_data_new, cabconv.cab_sr);
+     cabconv.configure(cabconv.cab_count, cabconv.cab_data, cabconv.cab_sr);
      while (!cabconv.checkstate());
      if(!cabconv.start(prio, SCHED_FIFO))
         printf("cabinet convolver update buffersize fail\n");
+
      ampconv.cleanup();
      ampconv.set_samplerate(s_rate);
      ampconv.set_buffersize(bufsize);
-     float pre_irdata_c[contrast_ir_desc.ir_count];
-     ampf.compute(contrast_ir_desc.ir_count,contrast_ir_desc.ir_data, pre_irdata_c, alevel_);
-     ampconv.configure(contrast_ir_desc.ir_count, pre_irdata_c, contrast_ir_desc.ir_sr);
+     ampconv.configure(contrast_ir_desc.ir_count, contrast_ir_desc.ir_data, contrast_ir_desc.ir_sr);
      while (!ampconv.checkstate());
      if(!ampconv.start(prio, SCHED_FIFO))
         printf("presence convolver update buffersize fail\n");
-  }
-  if (cab_changed() || change_cab())
+   }
+  if (cab_changed())
     {
       if (cabconv.is_runnable())
         {
@@ -362,6 +393,7 @@ void GxPluginMono::do_work_mono()
         cabconv.set_samplerate(s_rate);
         cabconv.set_buffersize(bufsize);
         cabconv.configure(cabconv.cab_count, cabconv.cab_data, cabconv.cab_sr);
+        //printf("cabconv.changed.\n");
       }
       float cab_irdata_c[cabconv.cab_count];
       float adjust_1x8 = 1;
@@ -374,6 +406,7 @@ void GxPluginMono::do_work_mono()
       if(!cabconv.start(prio, SCHED_FIFO))
         printf("cabinet convolver disabled\n");
       update_cab();
+      //printf("cabinet convolver updated\n");
     } // else printf("cabinet convolver disabled\n");
     }
   if (pre_changed())
@@ -391,15 +424,18 @@ void GxPluginMono::do_work_mono()
       if(!ampconv.start(prio, SCHED_FIFO))
         printf("presence convolver disabled\n");
       update_pre();
+      //printf("presence convolver updated\n");
     }
-  _execute.store(false, std::memory_order_release);
+  update_val();
+  atomic_set(&schedule_wait,0);
 }
 
 void GxPluginMono::init_dsp_mono(uint32_t rate, uint32_t bufsize_)
 {
+  AVOIDDENORMALS();
+
   bufsize = bufsize_;
   s_rate = rate;
-  bp.init_bypass(rate);
 #ifdef _POSIX_MEMLOCK_RANGE
   GX_LOCK::lock_rt_memory();
 #endif
@@ -449,7 +485,6 @@ void GxPluginMono::init_dsp_mono(uint32_t rate, uint32_t bufsize_)
       printf("convolver disabled\n");
       schedule_ok_ = 1.;
     }
-    _execute.store(false, std::memory_order_release);
 }
 
 
@@ -487,9 +522,6 @@ void GxPluginMono::connect_mono(uint32_t port,void* data)
     case AMP_INPUT:
       input = static_cast<float*>(data);
       break;
-    case BYPASS: 
-      bypass = static_cast<float*>(data); // , 0.0, 0.0, 1.0, 1.0 
-      break;
     default:
       break;
     }
@@ -497,43 +529,31 @@ void GxPluginMono::connect_mono(uint32_t port,void* data)
 
 void GxPluginMono::run_dsp_mono(uint32_t n_samples)
 {
-  if (n_samples< 1) return;
-  MXCSR.set_();
   cur_bufsize = n_samples;
   if (*(schedule_ok) != schedule_ok_) *(schedule_ok) = schedule_ok_;
   // run dsp
-  FAUSTFLOAT buf[n_samples];
-  // do inplace processing at default
-  if (output != input)
-    memcpy(output, input, n_samples*sizeof(float));
-  // check if bypass is pressed
-  if (!bp.pre_check_bypass(bypass, buf, input, n_samples)) {
 #ifndef __SSE__
-    wn->mono_audio(static_cast<int>(n_samples), input, input, wn);;
+  wn->mono_audio(static_cast<int>(n_samples), input, input, wn);;
 #endif
-    // run selected tube model
-    a_model_ = std::min(a_max, static_cast<uint32_t>(*(a_model)));
-    amplifier[a_model_]->mono_audio(static_cast<int>(n_samples), input, output, amplifier[a_model_]);
-    // run presence convolver
-    if (*(alevel) >= 1.0)
-        ampconv.run_static(n_samples, &ampconv, output);
-    // run selected tonestack
-    t_model_ =  static_cast<uint32_t>(*(t_model));
-    if (t_model_<=t_max)
-      tonestack[t_model_]->mono_audio(static_cast<int>(n_samples), output, output, tonestack[t_model_]);
-    // run selected cabinet convolver
-    cabconv.run_static(n_samples, &cabconv, output);
-  }
-  bp.post_check_bypass(buf, output, n_samples);
+  // run selected tube model
+  a_model_ = min(a_max, static_cast<uint32_t>(*(a_model)));
+  amplifier[a_model_]->mono_audio(static_cast<int>(n_samples), input, output, amplifier[a_model_]);
+  // run presence convolver
+  ampconv.run_static(n_samples, &ampconv, output);
+  // run selected tonestack
+  t_model_ =  static_cast<uint32_t>(*(t_model));
+  if (t_model_<=t_max)
+    tonestack[t_model_]->mono_audio(static_cast<int>(n_samples), output, output, tonestack[t_model_]);
+  // run selected cabinet convolver
+  cabconv.run_static(n_samples, &cabconv, output);
 
-  MXCSR.reset_();
   // work ?
-  if (!_execute.load(std::memory_order_acquire) && (val_changed() || buffsize_changed())) 
+  if (!atomic_get(schedule_wait) && (val_changed() || buffsize_changed())) 
     {
       clevel_ = (*clevel);
       alevel_ = (*alevel);
       c_model_= (*c_model);
-      _execute.store(true, std::memory_order_release);
+      atomic_set(&schedule_wait,1);
       schedule->schedule_work(schedule->handle, sizeof(bool), &doit);
     }
 }
@@ -593,7 +613,7 @@ LV2_Worker_Status GxPluginMono::work_response(LV2_Handle  instance,
               uint32_t    size,
               const void* data)
 {
-  //printf("worker response.\n");
+  //printf("worker respose.\n");
   return LV2_WORKER_SUCCESS;
 }
 
@@ -632,7 +652,7 @@ LV2_Handle GxPluginMono::instantiate(const LV2_Descriptor*     descriptor,
   if (!self->schedule)
     {
       fprintf(stderr, "Missing feature work:schedule.\n");
-      self->_execute.store(true, std::memory_order_release);
+      atomic_set(&self->schedule_wait,1);
       self->schedule_ok_ = 1.;
       //delete self;
       //return NULL;
@@ -642,10 +662,14 @@ LV2_Handle GxPluginMono::instantiate(const LV2_Descriptor*     descriptor,
   if (!self->map)
     {
       fprintf(stderr, "Missing feature uri:map.\n");
+      //atomic_set(&self->schedule_wait,1);
+      //self->schedule_ok_ = 1.;
     }
   else if (!options)
     {
       fprintf(stderr, "Missing feature options.\n");
+      //atomic_set(&self->schedule_wait,1);
+     // self->schedule_ok_ = 1.;
     }
   else
     {
@@ -673,6 +697,8 @@ LV2_Handle GxPluginMono::instantiate(const LV2_Descriptor*     descriptor,
       if (bufsize == 0)
         {
           fprintf(stderr, "No maximum buffer size given.\n");
+          //atomic_set(&self->schedule_wait,1);
+          //self->schedule_ok_ = 1.;
         }
       printf("using block size: %d\n", bufsize);
       self->schedule_ok_ = 0.;
@@ -735,6 +761,7 @@ const LV2_Descriptor GxPluginMono::descriptor =
   extension_data
 };
 
+extern "C"
 LV2_SYMBOL_EXPORT
 const LV2_Descriptor*
 lv2_descriptor(uint32_t index)
