@@ -23,11 +23,10 @@
  */
 
 #include <errno.h>              // NOLINT
-#ifndef GUITARIX_AS_PLUGIN
 #include <jack/statistics.h>    // NOLINT
 #include <jack/jack.h>          // NOLINT
 #include <jack/thread.h>        // NOLINT
-#endif
+
 #include "engine.h"           // NOLINT
 
 #ifdef HAVE_JACK_SESSION
@@ -36,7 +35,7 @@
 
 
 namespace gx_jack {
-#ifndef GUITARIX_AS_PLUGIN
+
 /****************************************************************
  ** class GxJack
  ****************************************************************/
@@ -96,41 +95,6 @@ static inline bool rt_watchdog_check_alive(unsigned int bs, unsigned int sr) {
 }
 
 
-/****************************************************************
- ** class MidiCC
- */
-
-MidiCC::MidiCC(gx_engine::GxEngine& engine_)
-    : engine(engine_) {
-    for (int i = 0; i < max_midi_cc_cnt; i++) {
-        send_cc[i] = false;
-    }
-}
-
-bool MidiCC::send_midi_cc(int _cc, int _pg, int _bgn, int _num) {
-    int c = engine.controller_map.get_midi_channel();
-    if (c) _cc |=c-1;
-    for(int i = 0; i < max_midi_cc_cnt; i++) {
-        if (send_cc[i].load(std::memory_order_acquire)) {
-            if (cc_num[i] == _cc && pg_num[i] == _pg &&
-                bg_num[i] == _bgn && me_num[i] == _num)
-                return true;
-        } else if (!send_cc[i].load(std::memory_order_acquire)) {
-            cc_num[i] = _cc;
-            pg_num[i] = _pg;
-            bg_num[i] = _bgn;
-            me_num[i] = _num;
-            send_cc[i].store(true, std::memory_order_release);
-            return true;
-        }
-    }
-#ifndef NDEBUG
-    cerr << "Internal error: MidiCC overflow" << endl;
-    assert(false);
-#endif
-    return false;
-}
-
 /****************************************************************
  ** GxJack ctor, dtor
  */
@@ -141,7 +105,6 @@ GxJack::GxJack(gx_engine::GxEngine& engine_)
       jack_is_down(false),
       jack_is_exit(true),
       bypass_insert(false),
-      mmessage(engine_),
 #ifdef HAVE_JACK_SESSION
       session_event(0),
       session_event_ins(0),
@@ -167,8 +130,8 @@ GxJack::GxJack(gx_engine::GxEngine& engine_)
       session(),
       session_ins(),
       shutdown(),
-      connection(),
-      single_client(false) {
+      connection() {
+	for(int i = 0;i<5;i++) mmessage.send_cc[i] = false;
     connection_queue.new_data.connect(sigc::mem_fun(*this, &GxJack::fetch_connection_data));
     client_change_rt.connect(client_change);
     GxExit::get_instance().signal_exit().connect(
@@ -218,7 +181,6 @@ void GxJack::read_connections(gx_system::JsonParser& jp) {
             jp.skip_object();
             continue;
         }
-	i->clear();
         jp.next(gx_system::JsonParser::begin_array);
         while (jp.peek() == gx_system::JsonParser::value_string) {
             jp.next();
@@ -260,10 +222,8 @@ void GxJack::write_connections(gx_system::JsonWriter& w) {
     write_jack_port_connections(w, "output2", ports.output2);
     write_jack_port_connections(w, "midi_input", ports.midi_input);
     write_jack_port_connections(w, "midi_output", ports.midi_output);
-    if (!single_client) {
     write_jack_port_connections(w, "insert_out", ports.insert_out, true);
     write_jack_port_connections(w, "insert_in", ports.insert_in, true);
-    }
     w.end_object(true);
 }
 
@@ -271,23 +231,14 @@ void GxJack::write_connections(gx_system::JsonWriter& w) {
 /****************************************************************
  ** client connection init and cleanup
  */
-int GxJack::is_power_of_two (unsigned int x)
-{
-    return ((x != 0) && ((x & (~x + 1)) == x));
-}
 
 // ----- pop up a dialog for starting jack
 bool GxJack::gx_jack_init(bool startserver, int wait_after_connect, const gx_system::CmdlineOptions& opt) {
     AVOIDDENORMALS();
-    single_client = opt.get_jack_single();
     int jackopt = (startserver ? JackNullOption : JackNoStartServer);
     client_instance = opt.get_jack_instancename();
     if (client_instance.empty()) {
-    if (!single_client) {
-        client_instance = get_default_instancename();
-    } else {
-        client_instance = "guitarix";
-    }
+	client_instance = get_default_instancename();
     } else {
 	jackopt |= JackUseExactName;
     }
@@ -300,11 +251,7 @@ bool GxJack::gx_jack_init(bool startserver, int wait_after_connect, const gx_sys
 
     //ports = JackPorts(); //FIXME
 
-    if (!single_client) {
-        client_name = client_instance + jack_amp_postfix;
-    } else {
-        client_name = client_instance;
-    }
+    client_name = client_instance + jack_amp_postfix;
     client_insert_name = client_instance + jack_fx_postfix;
     jack_status_t jackstat;
 #ifdef HAVE_JACK_SESSION
@@ -330,7 +277,7 @@ bool GxJack::gx_jack_init(bool startserver, int wait_after_connect, const gx_sys
     }
 #endif
     // ----- only start the insert gxjack.client when the amp gxjack.client is true
-    if (client && !single_client) {
+    if (client) {
 	// it is maybe not the 1st gx_head instance ?
 	// session handler can change name without setting JackNameNotUnique in return status; jack bug??
 	// this code depends on jackd only appending a suffix to make a client name unique
@@ -404,18 +351,10 @@ bool GxJack::gx_jack_init(bool startserver, int wait_after_connect, const gx_sys
 	boost::format(_("The jack sample rate is %1%/sec")) % jack_sr);
 
     jack_bs = jack_get_buffer_size(client); // jack buffer size
-	if (!is_power_of_two(jack_bs)) {
-    gx_print_warning(
-	_("Jack init"),
-	boost::format(_("The jack buffer size is %1%/frames is not power of two, Convolver won't run"))
-	% jack_bs);
-	} else {
     gx_print_info(
 	_("Jack init"),
 	boost::format(_("The jack buffer size is %1%/frames ... "))
 	% jack_bs);
-	}
-		
 	// create buffer to bypass the insert ports
     insert_buffer = new float[jack_bs];
     
@@ -426,12 +365,7 @@ bool GxJack::gx_jack_init(bool startserver, int wait_after_connect, const gx_sys
 	gx_jack_init_port_connection(opt);
     }
     set_jack_exit(false);
-	if (jack_sr > 96000) {
-    gx_print_fatal(
-		    _("Jack Init"),
-		    _("Sample rates above 96kHz ain't be supported"));
-		return false;
-	}
+
     return true;
 }
 
@@ -468,26 +402,19 @@ void GxJack::gx_jack_cleanup() {
     set_jack_exit(true);
     engine.set_stateflag(gx_engine::GxEngine::SF_INITIALIZING);
     jack_deactivate(client);
-    if (!single_client) jack_deactivate(client_insert);
+    jack_deactivate(client_insert);
     jack_port_unregister(client, ports.input.port);
     jack_port_unregister(client, ports.midi_input.port);
-    if (!single_client) {
-        jack_port_unregister(client, ports.insert_out.port);
-    } else {
-        jack_port_unregister(client, ports.output1.port);
-        jack_port_unregister(client, ports.output2.port);
-    }
+    jack_port_unregister(client, ports.insert_out.port);
 #if defined(USE_MIDI_OUT) || defined(USE_MIDI_CC_OUT)
     jack_port_unregister(client, ports.midi_output.port);
 #endif
-    if (!single_client) {
-        jack_port_unregister(client_insert, ports.insert_in.port);
-        jack_port_unregister(client_insert, ports.output1.port);
-        jack_port_unregister(client_insert, ports.output2.port);
-    }
+    jack_port_unregister(client_insert, ports.insert_in.port);
+    jack_port_unregister(client_insert, ports.output1.port);
+    jack_port_unregister(client_insert, ports.output2.port);
     jack_client_close(client);
     client = 0;
-    if (!single_client) jack_client_close(client_insert);
+    jack_client_close(client_insert);
     client_insert = 0;
     delete[] insert_buffer;
     insert_buffer = NULL;
@@ -569,7 +496,6 @@ void GxJack::gx_jack_init_port_connection(const gx_system::CmdlineOptions& opt)
         }
     }
 
-    if (!single_client) {
     // set autoconnect to user playback ports
     if (opt.get_jack_output(0).empty() && opt.get_jack_output(1).empty()) {
         list<string>& l1 = ports.output1.conn;
@@ -592,32 +518,6 @@ void GxJack::gx_jack_init_port_connection(const gx_system::CmdlineOptions& opt)
 			 opt.get_jack_output(1).c_str());
 	}
     }
-    
-    } else {
-// set autoconnect to user playback ports
-    if (opt.get_jack_output(0).empty() && opt.get_jack_output(1).empty()) {
-        list<string>& l1 = ports.output1.conn;
-        for (list<string>::iterator i = l1.begin(); i != l1.end(); ++i) {
-            jack_connect(client, jack_port_name(ports.output1.port), i->c_str());
-        }
-        list<string>& l2 = ports.output2.conn;
-        for (list<string>::iterator i = l2.begin(); i != l2.end(); ++i) {
-            jack_connect(client, jack_port_name(ports.output2.port), i->c_str());
-        }
-    } else {
-	if (!opt.get_jack_output(0).empty()) {
-	    jack_connect(client,
-			 jack_port_name(ports.output1.port),
-			 opt.get_jack_output(0).c_str());
-	}
-	if (!opt.get_jack_output(1).empty()) {
-	    jack_connect(client,
-			 jack_port_name(ports.output2.port),
-			 opt.get_jack_output(1).c_str());
-	}
-    }
-        
-    }
 
 #if defined(USE_MIDI_OUT) || defined(USE_MIDI_CC_OUT)
     // autoconnect midi output port
@@ -627,7 +527,6 @@ void GxJack::gx_jack_init_port_connection(const gx_system::CmdlineOptions& opt)
     }
 #endif
 
-    if (!single_client) {
     // autoconnect to insert ports
     list<string>& lins_in = ports.insert_in.conn;
     list<string>& lins_out = ports.insert_out.conn;
@@ -656,7 +555,6 @@ void GxJack::gx_jack_init_port_connection(const gx_system::CmdlineOptions& opt)
         jack_connect(client_insert, jack_port_name(ports.insert_out.port),
 		     (client_insert_name+":in_0").c_str());
     }
-    }
 }
 
 
@@ -670,16 +568,14 @@ void GxJack::gx_jack_callbacks() {
     jack_set_xrun_callback(client, gx_jack_xrun_callback, this);
     jack_set_sample_rate_callback(client, gx_jack_srate_callback, this);
     jack_on_shutdown(client, shutdown_callback_client, this);
-    if (!single_client) {
-        jack_on_shutdown(client_insert, shutdown_callback_client_insert, this);
-    }
+    jack_on_shutdown(client_insert, shutdown_callback_client_insert, this);
     jack_set_buffer_size_callback(client, gx_jack_buffersize_callback, this);
     jack_set_port_registration_callback(client, gx_jack_portreg_callback, this);
     jack_set_port_connect_callback(client, gx_jack_portconn_callback, this);
 #ifdef HAVE_JACK_SESSION
     if (jack_set_session_callback_fp) {
         jack_set_session_callback_fp(client, gx_jack_session_callback, this);
-        if (!single_client) jack_set_session_callback_fp(client_insert, gx_jack_session_callback_ins, this);
+        jack_set_session_callback_fp(client_insert, gx_jack_session_callback_ins, this);
     }
 #endif
 
@@ -688,15 +584,8 @@ void GxJack::gx_jack_callbacks() {
 	client, "in_0", JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
     ports.midi_input.port = jack_port_register(
 	client, "midi_in_1", JACK_DEFAULT_MIDI_TYPE, JackPortIsInput, 0);
-    if (!single_client) {
-        ports.insert_out.port = jack_port_register(
-        client, "out_0", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
-    } else {
-        ports.output1.port = jack_port_register(
-        client, "out_0", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
-        ports.output2.port = jack_port_register(
-        client, "out_1", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
-    }
+    ports.insert_out.port = jack_port_register(
+	client, "out_0", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
 #if defined(USE_MIDI_OUT) || defined(USE_MIDI_CC_OUT)
     ports.midi_output.port = jack_port_register(
 	client, "midi_out_1", JACK_DEFAULT_MIDI_TYPE, JackPortIsOutput, 0);
@@ -704,30 +593,26 @@ void GxJack::gx_jack_callbacks() {
     ports.midi_output.port = 0;
 #endif
 
-    if (!single_client) {
     // register ports for gx_amp_fx
-        ports.insert_in.port = jack_port_register(
-          client_insert, "in_0", JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
-        ports.output1.port = jack_port_register(
-          client_insert, "out_0", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
-        ports.output2.port = jack_port_register(
-          client_insert, "out_1", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
-    }
+    ports.insert_in.port = jack_port_register(
+	client_insert, "in_0", JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
+    ports.output1.port = jack_port_register(
+	client_insert, "out_0", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+    ports.output2.port = jack_port_register(
+	client_insert, "out_1", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
 
     engine.init(jack_sr, jack_bs, SCHED_FIFO,
 		jack_client_real_time_priority(client));
     jack_set_process_callback(client, gx_jack_process, this);
-    if (!single_client) jack_set_process_callback(client_insert, gx_jack_insert_process, this);
+    jack_set_process_callback(client_insert, gx_jack_insert_process, this);
     if (jack_activate(client) != 0) {
         gx_print_fatal(
 	    _("Jack Activation"),
 	    string(_("Can't activate JACK gx_amp client")));
     }
-    if (!single_client) {
-        if (jack_activate(client_insert) != 0) {
-            gx_print_fatal(_("Jack Activation"),
-                        string(_("Can't activate JACK gx_amp_fx client")));
-        }
+    if (jack_activate(client_insert) != 0) {
+        gx_print_fatal(_("Jack Activation"),
+                       string(_("Can't activate JACK gx_amp_fx client")));
     }
 }
 
@@ -737,12 +622,27 @@ void GxJack::gx_jack_callbacks() {
  */
 
 void __rt_func GxJack::process_midi_cc(void *buf, jack_nframes_t nframes) {
-    // midi CC output processing
-    for (int i = mmessage.next(); i >= 0; i = mmessage.next(i)) {
-        unsigned char* midi_send = jack_midi_event_reserve(buf, i, mmessage.size(i));
-        if (midi_send) {
-            mmessage.fill(midi_send, i);
-        }
+	// midi CC output processing
+	for(int i = 0;i<5;i++) {
+		if (mmessage.send_cc[i]) {
+			unsigned char* midi_send = jack_midi_event_reserve(buf, i, mmessage.me_num[i]);
+
+			if (midi_send) {
+				if (mmessage.me_num[i] == 2) {
+					// program value
+					midi_send[1] =  mmessage.pg_num[i];
+					// controller+ channel
+					midi_send[0] = mmessage.cc_num[i] | 0;
+				} else if (mmessage.me_num[i] == 3) {
+					midi_send[2] =  mmessage.bg_num[i];
+					// program value
+					midi_send[1] =  mmessage.pg_num[i];
+					// controller+ channel
+					midi_send[0] = mmessage.cc_num[i] | 0;
+				}
+			}
+			mmessage.send_cc[i] = false;
+		}
     }
 }
 
@@ -779,19 +679,14 @@ int __rt_func GxJack::gx_jack_process(jack_nframes_t nframes, void *arg) {
 	}
 	self.transport_state = jack_transport_query (self.client, &self.current);
         // gx_head DSP computing
-    float *obuf = self.insert_buffer;
-    if (!self.single_client) {
-        obuf = get_float_buf(self.ports.insert_out.port, nframes);
-    } 
+    float *obuf = get_float_buf(self.ports.insert_out.port, nframes);
 	self.engine.mono_chain.process(
 	    nframes,
 	    get_float_buf(self.ports.input.port, nframes),
 	    obuf);
 
-    if (self.bypass_insert && !self.single_client) {
+    if (self.bypass_insert) 
         memcpy(self.insert_buffer, obuf, nframes*sizeof(float));
-    }
-
         // midi input processing
 	if (self.ports.midi_input.port) {
 	    self.engine.controller_map.compute_midi_in(
@@ -809,9 +704,6 @@ int __rt_func GxJack::gx_jack_process(jack_nframes_t nframes, void *arg) {
 
     gx_system::measure_pause();
     self.engine.mono_chain.post_rt_finished();
-    if (self.single_client) {
-        self.gx_jack_insert_process(nframes, arg);
-    }
     return 0;
 }
 
@@ -826,7 +718,7 @@ int __rt_func GxJack::gx_jack_insert_process(jack_nframes_t nframes, void *arg)
 	}
         // gx_head DSP computing
     float *ibuf = NULL;
-    if (!self.bypass_insert && !self.single_client) {
+    if (!self.bypass_insert) {
 	    ibuf = get_float_buf(self.ports.insert_in.port, nframes);
 	} else {
 	    ibuf = self.insert_buffer;
@@ -938,6 +830,19 @@ void GxJack::gx_jack_portconn_callback(jack_port_id_t a, jack_port_id_t b, int c
  ** callbacks: portreg, buffersize, samplerate, shutdown, xrun
  */
 
+void GxJack::send_midi_cc(int _cc, int _pg, int _bgn, int _num) {
+	for(int i = 0;i<5;i++) {
+		if (!mmessage.send_cc[i]) {
+			mmessage.send_cc[i] = true;
+			mmessage.cc_num[i] = _cc;
+			mmessage.pg_num[i] = _pg;
+			mmessage.bg_num[i] = _bgn;
+			mmessage.me_num[i] = _num;
+			return;
+		}
+	}
+}
+
 // ----- fetch available jack ports other than gx_head ports
 // jackd1: RT process thread
 // jackd2: not RT thread
@@ -1000,12 +905,10 @@ void GxJack::shutdown_callback_client(void *arg) {
 	self.client = 0;
 	self.client_change_rt();
     }
-    if (!self.single_client) {
     if (self.client_insert) {
 	jack_client_close(self.client_insert);
 	self.client_insert = 0;
     }
-    }
     self.gx_jack_shutdown_callback();
 }
 
@@ -1124,6 +1027,6 @@ void GxJack::gx_jack_session_callback_ins(jack_session_event_t *event, void *arg
     }
     self.session_ins();
 }
-#endif // HAVE_JACK_SESSION
-#endif // GUITARIX_AS_PLUGIN
+#endif
+
 } /* end of gx_jack namespace */
