@@ -477,7 +477,8 @@ private:
 	static void remove_instance(LadspaGuitarix* i);
 	static void preset_change();
     };
-public:
+protected:
+    friend class PresetLoader;
     friend class LibMonitor;
     pthread_t last_thread_id;
     int jack_bs;
@@ -728,7 +729,6 @@ void LadspaGuitarix::PresetLoader::remove_instance(LadspaGuitarix* i) {
 class MonoEngine: public EngineControl {
 private:
     gx_resample::BufferResampler resamp;
-    gx_resample::FixedRateResampler smp;
     void load_static_plugins();
     void sr_changed(unsigned int);
     void bs_changed(unsigned int);
@@ -741,7 +741,6 @@ public:
     ModuleSelectorFromList wah;
     ModuleSelectorFromList tonestack;
     ModuleSelectorFromList ampstack;
-    ModuleSelectorFromList poweramps;
     // internal audio modules
     NoiseGate noisegate;
     ConvolverMonoAdapter mono_convolver;
@@ -905,19 +904,9 @@ static const char* ampstack_groups[] = {
     0
 };
 
-static plugindef_creator builtin_poweramp_plugins[] = {
-    gx_poweramps::champ::plugin,
-    gx_poweramps::epiphone::plugin,
-    gx_poweramps::princeton::plugin,
-    gx_poweramps::orangedarkterror::plugin,
-    gx_poweramps::supersonic::plugin,
-    0
-};
-
 MonoEngine::MonoEngine(const string& plugin_dir, const string& loop_dir, ParameterGroups& groups)
     : EngineControl(),
       resamp(),
-      smp(),
       // ModuleSelector's
       crybaby(
 	  *this, "crybaby", N_("Crybaby"), "", builtin_crybaby_plugins,
@@ -932,17 +921,14 @@ MonoEngine::MonoEngine(const string& plugin_dir, const string& loop_dir, Paramet
       ampstack(
 	  *this, "ampstack", "?Tube", "", builtin_amp_plugins,
 	  "tube.select", _("select"), 0, ampstack_groups),
-      poweramps(
-	  *this, "poweramp", _("PowerAmp"), N_("Guitar Effects"), builtin_poweramp_plugins,
-	  "poweramp.mode", _("select"), 0, 0, PGN_POST_PRE),
       // internal audio modules
       noisegate(),
-      mono_convolver(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync)),
+      mono_convolver(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), get_param()),
       cabinet(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp),
       preamp(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp),
       contrast(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp),
       loop(get_param(), sigc::mem_fun(mono_chain, &MonoModuleChain::sync), loop_dir),
-      record(*this, 1), detune(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync)) {
+      record(*this, 1), detune(get_param(), *this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync)) {
 
     mono_convolver.set_sync(true);
     cabinet.set_sync(true);
@@ -961,7 +947,6 @@ MonoEngine::MonoEngine(const string& plugin_dir, const string& loop_dir, Paramet
     add_selector(crybaby);
     add_selector(wah);
     add_selector(tonestack);
-    add_selector(poweramps);
 
     registerParameter(groups);
 
@@ -1014,9 +999,8 @@ void MonoEngine::load_static_plugins() {
     // dynamic rack modules
     // builtin 
     pl.add(builtin_crybaby_plugins,               PLUGIN_POS_RACK, PGN_ALTERNATIVE);
-    pl.add(builtin_wah_plugins,                   PLUGIN_POS_RACK, PGN_ALTERNATIVE);
+    pl.add(builtin_wah_plugins,               PLUGIN_POS_RACK, PGN_ALTERNATIVE);
     pl.add(builtin_tonestack_plugins,             PLUGIN_POS_RACK, PGN_ALTERNATIVE);
-    pl.add(builtin_poweramp_plugins,              PLUGIN_POS_RACK, PGN_ALTERNATIVE);
 
     // mono
     pl.add(gx_effects::gain::plugin(),            PLUGIN_POS_RACK, PGN_GUI);
@@ -1024,16 +1008,15 @@ void MonoEngine::load_static_plugins() {
     pl.add(gx_effects::highbooster::plugin(),     PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::selecteq::plugin(),        PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&crybaby.plugin,                       PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&poweramps.plugin,                     PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&wah.plugin,                           PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(&wah.plugin,                       PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&loop.plugin,                          PLUGIN_POS_RACK,  PGN_GUI);
     pl.add(&record.plugin,                        PLUGIN_POS_RACK,  PGN_GUI);
     pl.add(&detune.plugin,                        PLUGIN_POS_RACK,  PGN_GUI);
     pl.add(gx_effects::gx_distortion::plugin(),   PLUGIN_POS_RACK, PGN_GUI);
     pl.add(pluginlib::ts9sim::plugin(),           PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(pluginlib::aclipper::plugin(),         PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(pluginlib::mxrdist::plugin(),          PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(pluginlib::bossds1::plugin(),          PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(pluginlib::aclipper::plugin(),           PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(pluginlib::mxrdist::plugin(),           PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(pluginlib::bossds1::plugin(),           PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::impulseresponse::plugin(), PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::compressor::plugin(),      PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::overdrive::plugin(),       PLUGIN_POS_RACK, PGN_GUI);
@@ -1079,7 +1062,7 @@ void MonoEngine::load_static_plugins() {
  ** class LadspaGuitarixMono
  */
 
-class LadspaGuitarixMono {
+class LadspaGuitarixMono: LadspaGuitarix {
 private:
 
     enum {  // ladspa ports
@@ -1137,7 +1120,6 @@ private:
 
     MonoEngine engine;
     ControlParameter control_parameter;
-    LadspaGuitarix ladspa_guitarix;
     ReBuffer rebuffer;
     LADSPA_Data * volume_port;
     FloatParameter& volume_param;
@@ -1157,10 +1139,10 @@ public:
 };
 
 LadspaGuitarixMono::LadspaGuitarixMono(unsigned long sr)
-    : engine(Glib::build_filename(Glib::get_user_config_dir(), "guitarix/plugins/"),
+    : LadspaGuitarix(engine, 0, &engine.mono_convolver, control_parameter, "LADSPA_GUITARIX_MONO_PRESET"),
+      engine(Glib::build_filename(Glib::get_user_config_dir(), "guitarix/plugins/"),
        Glib::build_filename(Glib::get_user_config_dir(), "guitarix/pluginpresets/loops/"), get_group_table()),
       control_parameter(GUITARIX_PARAM_COUNT),
-      ladspa_guitarix(engine, 0, &engine.mono_convolver, control_parameter, "LADSPA_GUITARIX_MONO_PRESET"),
       rebuffer(),
       volume_port(),
       volume_param(engine.get_param()["amp.out_ladspa"].getFloat()),
@@ -1181,7 +1163,7 @@ const LADSPA_Descriptor *LadspaGuitarixMono::ladspa_descriptor() {
 void LadspaGuitarixMono::activateGuitarix(LADSPA_Handle Instance) {
     LadspaGuitarixMono& self = *static_cast<LadspaGuitarixMono*>(Instance);
     int policy, prio, bufsize;
-    bufsize = self.ladspa_guitarix.activate(&policy, &prio);
+    bufsize = self.activate(&policy, &prio);
     self.rebuffer.set_bufsize(bufsize);
     self.engine.set_buffersize(bufsize);
     gx_print_info(
@@ -1190,14 +1172,14 @@ void LadspaGuitarixMono::activateGuitarix(LADSPA_Handle Instance) {
 	% Instance % self.engine.get_samplerate() % bufsize % prio);
     self.engine.init(self.engine.get_samplerate(), bufsize, policy, prio);
     self.engine.mono_chain.set_stopped(true);
-    self.ladspa_guitarix.load();
+    self.load();
     self.engine.mono_chain.set_stopped(false);
     self.engine.mono_chain.start_ramp_up();
 }
 
 void LadspaGuitarixMono::runGuitarix(LADSPA_Handle Instance, unsigned long SampleCount) {
     LadspaGuitarixMono& self = *static_cast<LadspaGuitarixMono*>(Instance);
-    self.ladspa_guitarix.prepare_run();
+    self.prepare_run();
     self.volume_param.set(*self.volume_port);
     if (self.rebuffer.get_bufsize()) {
 	self.rebuffer.set(SampleCount, self.input_buffer, self.output_buffer);
@@ -1224,26 +1206,26 @@ void LadspaGuitarixMono::connectPortToGuitarix(
 	self->output_buffer = DataLocation;
 	break;
     case GUITARIX_PRESET:
-	self->ladspa_guitarix.preset_num_port = DataLocation;
+	self->preset_num_port = DataLocation;
 	break;
     case GUITARIX_VOLUME:
 	self->volume_port = DataLocation;
 	break;
     case GUITARIX_NO_BUFFER:
-	self->ladspa_guitarix.no_buffer_port = DataLocation;
+	self->no_buffer_port = DataLocation;
 	break;
     case GUITARIX_BUFFERSIZE:
-	self->ladspa_guitarix.buffersize_port = DataLocation;
+	self->buffersize_port = DataLocation;
 	break;
     case GUITARIX_NO_RT_MODE:
-	self->ladspa_guitarix.no_rt_mode_port = DataLocation;
+	self->no_rt_mode_port = DataLocation;
 	break;
     case GUITARIX_PRIORITY:
-	self->ladspa_guitarix.priority_port = DataLocation;
+	self->priority_port = DataLocation;
 	break;
     case GUITARIX_LATENCY:
-	self->ladspa_guitarix.latency_port = DataLocation;
-	*self->ladspa_guitarix.latency_port = 0;
+	self->latency_port = DataLocation;
+	*self->latency_port = 0;
 	break;
     default:
 	self->control_parameter.set_port(Port-GUITARIX_PARAM, DataLocation);
@@ -1253,7 +1235,7 @@ void LadspaGuitarixMono::connectPortToGuitarix(
 
 LADSPA_Handle LadspaGuitarixMono::instantiateGuitarix(
     const LADSPA_Descriptor * Descriptor, unsigned long SampleRate) {
-    LadspaGuitarix::start_presetloader();
+    start_presetloader();
     return new LadspaGuitarixMono(SampleRate);
 }
 
@@ -1528,7 +1510,7 @@ void StereoEngine::set_samplerate(unsigned int samplerate) {
 StereoEngine::StereoEngine(const string& plugin_dir, ParameterGroups& groups)
     : EngineControl(),
       // internal audio modules
-      stereo_convolver(*this, sigc::mem_fun(stereo_chain, &StereoModuleChain::sync)),
+      stereo_convolver(*this, sigc::mem_fun(stereo_chain, &StereoModuleChain::sync), get_param()),
       record_st(*this, 2) {
 
     stereo_convolver.set_sync(true);
@@ -1559,8 +1541,6 @@ void StereoEngine::load_static_plugins() {
 
     // rack stereo modules inserted here
 
-    pl.add(fizz_remover::plugin(),                PLUGIN_POS_END, PGN_MODE_NORMAL);
-    pl.add(hardlim::plugin(),                     PLUGIN_POS_END, PGN_MODE_NORMAL);
     pl.add(gx_outputlevel_ladspa::plugin(),       PLUGIN_POS_END);
 
     // * fx amp output *
@@ -1577,7 +1557,7 @@ void StereoEngine::load_static_plugins() {
     pl.add(gx_effects::tonecontroll::plugin(),    PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::digital_delay_st::plugin(),PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&stereo_convolver.plugin,              PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&record_st.plugin,                     PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(&record_st.plugin,                   PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::stereoverb::plugin(),      PLUGIN_POS_RACK, PGN_GUI);
     pl.add(pluginlib::zita_rev1::plugin(),        PLUGIN_POS_RACK);
     pl.add(pluginlib::vibe::plugin_stereo(),      PLUGIN_POS_RACK);
@@ -1594,7 +1574,7 @@ void StereoEngine::load_static_plugins() {
  ** class LadspaGuitarixStereo
  */
 
-class LadspaGuitarixStereo {
+class LadspaGuitarixStereo: LadspaGuitarix {
 private:
 
     enum {  // ladspa ports
@@ -1663,7 +1643,6 @@ private:
 
     StereoEngine engine;
     ControlParameter control_parameter;
-    LadspaGuitarix ladspa_guitarix;
     ReBuffer rebuffer;
     LADSPA_Data * volume_port;
     FloatParameter& volume_param;
@@ -1685,9 +1664,9 @@ public:
 };
 
 LadspaGuitarixStereo::LadspaGuitarixStereo(unsigned long sr)
-    : engine(Glib::build_filename(Glib::get_user_config_dir(), "guitarix/plugins/"), get_group_table()),
+    : LadspaGuitarix(engine, &engine.stereo_convolver, 0, control_parameter, "LADSPA_GUITARIX_STEREO_PRESET"),
+      engine(Glib::build_filename(Glib::get_user_config_dir(), "guitarix/plugins/"), get_group_table()),
       control_parameter(GUITARIX_PARAM_COUNT),
-      ladspa_guitarix(engine, &engine.stereo_convolver, 0, control_parameter, "LADSPA_GUITARIX_STEREO_PRESET"),
       rebuffer(),
       volume_port(),
       volume_param(engine.get_param()["amp.out_master_ladspa"].getFloat()),
@@ -1710,7 +1689,7 @@ const LADSPA_Descriptor *LadspaGuitarixStereo::ladspa_descriptor() {
 void LadspaGuitarixStereo::activateGuitarix(LADSPA_Handle Instance) {
     LadspaGuitarixStereo& self = *static_cast<LadspaGuitarixStereo*>(Instance);
     int policy, prio, bufsize;
-    bufsize = self.ladspa_guitarix.activate(&policy, &prio);
+    bufsize = self.activate(&policy, &prio);
     self.rebuffer.set_bufsize(bufsize);
     self.engine.set_buffersize(bufsize);
     gx_print_info(
@@ -1719,14 +1698,14 @@ void LadspaGuitarixStereo::activateGuitarix(LADSPA_Handle Instance) {
 	% Instance % self.engine.get_samplerate() % bufsize % prio);
     self.engine.init(self.engine.get_samplerate(), bufsize, policy, prio);
     self.engine.stereo_chain.set_stopped(true);
-    self.ladspa_guitarix.load();
+    self.load();
     self.engine.stereo_chain.set_stopped(false);
     self.engine.stereo_chain.start_ramp_up();
 }
 
 void LadspaGuitarixStereo::runGuitarix(LADSPA_Handle Instance, unsigned long SampleCount) {
     LadspaGuitarixStereo& self = *static_cast<LadspaGuitarixStereo*>(Instance);
-    self.ladspa_guitarix.prepare_run();
+    self.prepare_run();
     self.volume_param.set(*self.volume_port);
     if (self.rebuffer.get_bufsize()) {
 	self.rebuffer.set(SampleCount, self.input_buffer1, self.input_buffer2,
@@ -1761,26 +1740,26 @@ void LadspaGuitarixStereo::connectPortToGuitarix(
 	self->output_buffer2 = DataLocation;
 	break;
     case GUITARIX_PRESET:
-	self->ladspa_guitarix.preset_num_port = DataLocation;
+	self->preset_num_port = DataLocation;
 	break;
     case GUITARIX_VOLUME:
 	self->volume_port = DataLocation;
 	break;
     case GUITARIX_NO_BUFFER:
-	self->ladspa_guitarix.no_buffer_port = DataLocation;
+	self->no_buffer_port = DataLocation;
 	break;
     case GUITARIX_BUFFERSIZE:
-	self->ladspa_guitarix.buffersize_port = DataLocation;
+	self->buffersize_port = DataLocation;
 	break;
     case GUITARIX_NO_RT_MODE:
-	self->ladspa_guitarix.no_rt_mode_port = DataLocation;
+	self->no_rt_mode_port = DataLocation;
 	break;
     case GUITARIX_PRIORITY:
-	self->ladspa_guitarix.priority_port = DataLocation;
+	self->priority_port = DataLocation;
 	break;
     case GUITARIX_LATENCY:
-	self->ladspa_guitarix.latency_port = DataLocation;
-	*self->ladspa_guitarix.latency_port = 0;
+	self->latency_port = DataLocation;
+	*self->latency_port = 0;
 	break;
     default:
 	self->control_parameter.set_port(Port-GUITARIX_PARAM, DataLocation);
@@ -1790,7 +1769,7 @@ void LadspaGuitarixStereo::connectPortToGuitarix(
 
 LADSPA_Handle LadspaGuitarixStereo::instantiateGuitarix(
     const LADSPA_Descriptor * Descriptor, unsigned long SampleRate) {
-    LadspaGuitarix::start_presetloader();
+    start_presetloader();
     return new LadspaGuitarixStereo(SampleRate);
 }
 
